#include "TilemapComponent.h"
#include "../utils/Logging.h"
#include "../utils/FileDialog.h"
#include "../utils/utils.h"
#include <stb_image.h>
#include <cstdio>
#include <cstring>
#include <GLFW/glfw3.h>

static int g_selectedTileIndex = 0;


// Constructor: initialize with a default grid (optional)
TilemapComponent::TilemapComponent(Object *owner)
    : Component(owner)
{
    // For example, create an empty 10x10 tilemap
    SetGridSize(glm::ivec2(10, 10));
    // Optionally set a default tile size
    m_TileSize = glm::ivec2(32, 32);
}

// Set grid dimensions and resize tile array; initially all -1 (empty)
void TilemapComponent::SetGridSize(const glm::ivec2 &size)
{
    m_GridSize = size;
    m_Tiles.assign(size.x * size.y, -1);
}

// Set individual tile at (x, y)
void TilemapComponent::SetTile(int x, int y, int tileIndex)
{
    if (x < 0 || y < 0 || x >= m_GridSize.x || y >= m_GridSize.y)
        return;
    m_Tiles[y * m_GridSize.x + x] = tileIndex;
}

int TilemapComponent::GetTile(int x, int y) const
{
    if (x < 0 || y < 0 || x >= m_GridSize.x || y >= m_GridSize.y)
        return -1;
    return m_Tiles[y * m_GridSize.x + x];
}

void TilemapComponent::SetTileSize(const glm::ivec2 &size)
{
    m_TileSize = size;
}

// Set atlas (tileset) settings; also update dimensions
void TilemapComponent::SetAtlas(const std::string &path, int atlasTileWidth, int atlasTileHeight)
{
    m_AtlasPath = path;
    m_AtlasTileWidth = atlasTileWidth;
    m_AtlasTileHeight = atlasTileHeight;
    UpdateAtlasDimensions();
}

// Helper: load atlas image info using stbi_info to compute columns and rows
void TilemapComponent::UpdateAtlasDimensions()
{
    if (m_AtlasPath.empty())
        return;
    int texW, texH, comp;
    if (stbi_info(m_AtlasPath.c_str(), &texW, &texH, &comp))
    {
        m_AtlasCols = texW / m_AtlasTileWidth;
        m_AtlasRows = texH / m_AtlasTileHeight;
    }
    else
    {
        m_AtlasCols = m_AtlasRows = 0;
        Logger::LogError("Failed to get atlas info: %s", m_AtlasPath.c_str());
    }
}


bool TilemapComponent::IsTileFullyTransparent(int col, int row, int texW, int texH, unsigned char* imageData)
{
    int x0 = col * m_AtlasTileWidth;
    int y0 = row * m_AtlasTileHeight;

    for (int y = 0; y < m_AtlasTileHeight; ++y)
    {
        for (int x = 0; x < m_AtlasTileWidth; ++x)
        {
            int px = x0 + x;
            int py = y0 + y;
            int idx = (py * texW + px) * 4;
            if (imageData[idx + 3] > 0)
                return false;
        }
    }
    return true;
}



// Serialization: store grid size, tile size, tile data, and atlas settings.
void TilemapComponent::Save(YAML::Emitter &out) const
{
    out << YAML::BeginMap;
    out << YAML::Key << "type" << YAML::Value << "TilemapComponent";
    out << YAML::Key << "gridSize" << YAML::Value << std::vector<int>{m_GridSize.x, m_GridSize.y};
    out << YAML::Key << "tileSize" << YAML::Value << std::vector<int>{m_TileSize.x, m_TileSize.y};
    out << YAML::Key << "tiles" << YAML::Value << m_Tiles;
    out << YAML::Key << "atlasPath" << YAML::Value << m_AtlasPath;
    out << YAML::Key << "atlasTileSize" << YAML::Value << std::vector<int>{m_AtlasTileWidth, m_AtlasTileHeight};
    out << YAML::EndMap;
}

// Deserialization: load saved data
void TilemapComponent::Load(const YAML::Node &node)
{
    if (node["gridSize"])
    {
        auto vec = node["gridSize"].as<std::vector<int>>();
        if (vec.size() == 2)
            m_GridSize = glm::ivec2(vec[0], vec[1]);
    }
    if (node["tileSize"])
    {
        auto vec = node["tileSize"].as<std::vector<int>>();
        if (vec.size() == 2)
            m_TileSize = glm::ivec2(vec[0], vec[1]);
    }
    if (node["tiles"])
        m_Tiles = node["tiles"].as<std::vector<int>>();
    if (node["atlasPath"])
        m_AtlasPath = node["atlasPath"].as<std::string>();
    if (node["atlasTileSize"])
    {
        auto vec = node["atlasTileSize"].as<std::vector<int>>();
        if (vec.size() == 2)
        {
            m_AtlasTileWidth = vec[0];
            m_AtlasTileHeight = vec[1];
        }
    }
    // Update atlas dimensions after loading settings
    UpdateAtlasDimensions();
}

void TilemapComponent::DrawEditorUI()
{
    ImGui::Begin("Tilemap Editor");

    if (ImGui::BeginTabBar("TilemapEditorTabs"))
    {
        // --- Atlas Settings ---
        if (ImGui::BeginTabItem("Atlas Settings"))
        {
            ImGui::Text("Current Atlas: %s", m_AtlasPath.empty() ? "None" : m_AtlasPath.c_str());
            if (ImGui::Button("Import Atlas"))
            {
                std::string path = OpenFileDialog(FileDialogType::Images);
                if (!path.empty())
                {
                    m_AtlasPath = path;
                    UpdateAtlasDimensions();
                }
            }

            int atlasW = m_AtlasTileWidth, atlasH = m_AtlasTileHeight;
            if (ImGui::InputInt("Atlas Tile Width", &atlasW))
            {
                m_AtlasTileWidth = atlasW;
                UpdateAtlasDimensions();
            }
            if (ImGui::InputInt("Atlas Tile Height", &atlasH))
            {
                m_AtlasTileHeight = atlasH;
                UpdateAtlasDimensions();
            }

            ImGui::Text("Atlas Grid: %d x %d", m_AtlasCols, m_AtlasRows);
            ImGui::EndTabItem();
        }

        // --- Map Settings ---
        if (ImGui::BeginTabItem("Map Settings"))
        {
            ImGui::Text("Map Size: %d x %d", m_GridSize.x, m_GridSize.y);
            ImGui::Text("Tile Size: %d x %d", m_TileSize.x, m_TileSize.y);
            ImGui::Text("Tile Count: %zu", m_Tiles.size());

            
            ImGui::EndTabItem();
        }

        // --- Tile Tools ---
        if (ImGui::BeginTabItem("Tiles"))
        {

            ImGui::Separator();

            if (m_AtlasPath.empty())
            {
                ImGui::Text("No atlas loaded.");
            }
            else
            {
                int texW, texH, comp;
                unsigned char* imageData = stbi_load(m_AtlasPath.c_str(), &texW, &texH, &comp, 4);
                if (imageData)
                {
                    float uvTileW = float(m_AtlasTileWidth) / float(texW);
                    float uvTileH = float(m_AtlasTileHeight) / float(texH);
                    GLuint textureID = LoadTextureIfNeeded(m_AtlasPath);

                    const float tileDisplaySize = 48;
                    ImGui::BeginChild("TilesetView");
                    for (int row = 0; row < m_AtlasRows; ++row)
                    {
                        for (int col = 0; col < m_AtlasCols; ++col)
                        {
                            int tileIndex = row * m_AtlasCols + col;
                            float uvX = float(col) * uvTileW;
                            float uvY = float(row) * uvTileH;
                            ImVec2 uv0(uvX, uvY);
                            ImVec2 uv1(uvX + uvTileW, uvY + uvTileH);

                            char id[32];
                            sprintf(id, "##tile_%d", tileIndex);

                            bool isEmpty = IsTileFullyTransparent(col, row, texW, texH, imageData);
                            ImVec4 tint = isEmpty ? ImVec4(0.3f, 0.3f, 0.3f, 0.5f) : ImVec4(1, 1, 1, 1);
                            if (ImGui::ImageButton(id, (ImTextureID)(uintptr_t)textureID, ImVec2(tileDisplaySize, tileDisplaySize), uv0, uv1))
                            {
                                g_selectedTileIndex = tileIndex;
                            }

                            if (ImGui::IsItemHovered())
                                ImGui::SetTooltip("Tile %d", tileIndex);

                            ImGui::SameLine();
                        }
                        ImGui::NewLine();
                    }
                    stbi_image_free(imageData);
                    ImGui::EndChild();
                }
                else
                {
                    ImGui::Text("Failed to load image.");
                }

                ImGui::Text("Selected Tile: %d", g_selectedTileIndex);
            }

            ImGui::EndTabItem();
        }

        ImGui::EndTabBar();
    }

    ImGui::End();
}




