#include "Engine.h"
#include "Entitys/Object.h"
#include "Renderer.h"
#include "components/SpriteComponent.h"
#include "components/CameraComponent.h"
#include "components/LightComponent.h"
#include "components/TilemapComponent.h"
#include "components/ScriptComponent.h"
#include "components/PhysicsComponent.h"

#include "utils/FileDialog.h"
#include "utils/Logging.h"

#include "utils/EngineConfig.h"
#include "utils/GameObjectsList.h"
#include "utils/Profiler.h"
#include "utils/utils.h"

#include <GL/glew.h>
#include <GLFW/glfw3.h>
#include <imgui.h>
#include <imgui_impl_glfw.h>
#include <imgui_impl_opengl3.h>

#include <functional>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <vector>
#include <cmath>
#include <cstdio>
#include <algorithm>
#include <random>
#include <format>
#include <memory>
#include <filesystem>
#include <iostream>
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <commdlg.h>
#include <openssl/sha.h>
#include <fstream>
#include <json.hpp>
using json = nlohmann::json;

static std::shared_ptr<Object> selected = nullptr;
static bool playing = false;
static bool lastPlaying = false;

static std::vector<std::shared_ptr<Object>> pendingDeletion;

static glm::vec2 cameraPos = {0, 0};
static float cameraZoom = 1.0f;
static ImVec2 lastMousePos = {};

static glm::vec2 editorCameraSavedPos = {0.0f, 0.0f};
static float editorCameraSavedZoom = 1.0f;
static bool editorCameraWasSaved = false;

static std::shared_ptr<CameraComponent> m_activeCamera;

float g_fps = 0.0f;

static const std::string tempScenePath = "__tmp_scene.yaml";

GLFWwindow *window = nullptr;

Engine::Engine()
{

    Init();
}

Engine::~Engine()
{
    Shutdown();
}

void DrawProfilerTimelineBars(const ProfileNode &node,
                              int depth,
                              float baseOffsetX,
                              float frameDuration,
                              ImVec2 origin)
{
    constexpr float rowHeight = 24.0f;
    constexpr float textPadding = 5.0f;

    constexpr size_t SEED = 9873;

    // compute bar rect
    float startRel = node.visualStartMs / frameDuration;
    float durRel = node.visualDurationMs / frameDuration;
    float x = origin.x + startRel * baseOffsetX;
    float width = std::fmax(1.0f, durRel * baseOffsetX);
    float y = origin.y + depth * rowHeight;

    ImVec2 barMin(x, y);
    ImVec2 barMax(x + width, y + rowHeight - 4.0f);

    // draw the bar
    size_t hash = std::hash<std::string>{}(node.name);
    ImU32 color = ImColor::HSV(((hash ^ SEED) % 1000) / 1000.0f, 0.6f, 0.85f);

    ImDrawList *draw = ImGui::GetWindowDrawList();
    draw->AddRectFilled(barMin, barMax, color, 3.0f);
    draw->AddRect(barMin, barMax, IM_COL32_BLACK, 3.0f);

    // how much room we have for text
    float availW = width - textPadding * 2;
    const char *textToDraw = nullptr;

    // 1) try the short "XX.XX ms"
    char timeBuf[16];
    int tn = std::snprintf(timeBuf, sizeof(timeBuf), "%.2f ms", node.visualDurationMs);
    if (tn > 0)
    {
        ImVec2 sz = ImGui::CalcTextSize(timeBuf);
        if (sz.x <= availW)
        {
            textToDraw = timeBuf;
        }
    }

    // 2) only if the short fits, see if full fits
    if (textToDraw == timeBuf)
    {
        char fullBuf[64];
        int fn = std::snprintf(fullBuf, sizeof(fullBuf),
                               "%s (%.2f ms)",
                               node.name.c_str(),
                               node.visualDurationMs);
        if (fn > 0)
        {
            ImVec2 fsz = ImGui::CalcTextSize(fullBuf);
            if (fsz.x <= availW)
            {
                textToDraw = fullBuf;
            }
        }
    }

    // 3) draw if we decided on something
    if (textToDraw)
    {
        draw->AddText(
            ImVec2(barMin.x + textPadding,
                   barMin.y),
            IM_COL32_WHITE,
            textToDraw);
    }

    // tooltip
    ImVec2 mouse = ImGui::GetMousePos();
    if (mouse.x >= barMin.x && mouse.x <= barMax.x &&
        mouse.y >= barMin.y && mouse.y <= barMax.y)
    {
        ImGui::SetTooltip(
            "%s\nCurrent: %.3f ms\nSmoothed: %.3f ms",
            node.name.c_str(),
            node.durationMs,
            node.visualDurationMs);
    }

    // recurse
    for (auto &child : node.children)
        DrawProfilerTimelineBars(child,
                                 depth + 1,
                                 baseOffsetX,
                                 frameDuration,
                                 origin);
}

static ProfileNode GetAveragedFrameCopy(const ProfileNode &src)
{
    ProfileNode averaged = src;
    averaged.visualStartMs = src.startMs;
    averaged.visualDurationMs = src.durationMs;

    averaged.children.clear();
    for (const auto &child : src.children)
        averaged.children.push_back(GetAveragedFrameCopy(child));

    return averaged;
}

void ShowProfilerTimeline()
{
    static ProfileNode cachedAveragedFrame;
    static double lastUpdateTime = 0.0;
    static double updateInterval = 0.25;
    static bool freezeView = false;

    const double now = ImGui::GetTime();
    const ProfileNode *latest = profiler.GetLatestFrame();

    ImGui::Begin("Profiler", nullptr, ImGuiWindowFlags_NoScrollbar);

    // --- Controls ---
    if (g_engineConfig.settings.profile_enabled)
    {
        ImGui::Checkbox("Freeze View", &freezeView);
        ImGui::SameLine();
        ImGui::SetNextItemWidth(100);
        float interval = static_cast<float>(updateInterval);
        if (ImGui::DragFloat("Update Interval", &interval, 0.05f, 0.05f, 5.0f))
            updateInterval = interval;

        updateInterval = std::clamp(updateInterval, 0.05, 5.0);

        // Export JSON button
        ImGui::SameLine();
        if (ImGui::Button("Export"))
        {
            json root = SerializeProfileNode(cachedAveragedFrame);
            std::ofstream out("profile_export.json");
            if (out.is_open())
            {
                out << root.dump(4);
                out.close();
                Logger::LogInfo("Profiler exported to profile_export.json");
            }
            else
            {
                Logger::LogError("Failed to write profiler export.");
            }
        }

        if (!freezeView && latest && (now - lastUpdateTime) >= updateInterval)
        {
            cachedAveragedFrame = GetAveragedFrameCopy(*latest);
            lastUpdateTime = now;
        }

        if (cachedAveragedFrame.durationMs <= 0.0f)
        {
            ImGui::Text("Waiting for profiler data...");
            ImGui::End();
            return;
        }

        const float rowHeight = 24.0f;
        const float timelineHeight = 400.0f;
        float frameDuration = cachedAveragedFrame.visualDurationMs;
        float timelineWidth = ImGui::GetContentRegionAvail().x * 0.95f;

        ImGui::BeginChild("TimelineScroll", ImVec2(0, timelineHeight));
        ImVec2 origin = ImGui::GetCursorScreenPos();

        DrawProfilerTimelineBars(cachedAveragedFrame, 0, timelineWidth, frameDuration, origin);

        ImGui::Dummy(ImVec2(timelineWidth, 32 * rowHeight));
        ImGui::EndChild();

        ImGui::Text("Frame Duration (Avg): %.2f ms | View Update Rate: %.2fs", frameDuration, updateInterval);
    }
    else
    {
        ImGui::Text("Profiling Disabled.");
    }

    ImGui::End();
}

void Engine::ShowDebugOverlay(float deltaTime)
{
    // === Performance Window ===
    static std::vector<float> fpsHistory;
    static const int maxFpsHistory = 100;
    static float fpsTimer = 0.0f;
    float fps = 1.0f / deltaTime;
    g_fps = fps;
    if (fpsHistory.size() >= maxFpsHistory)
        fpsHistory.erase(fpsHistory.begin());
    if (fpsTimer >= 0.05f)
    {
        fpsTimer = 0;
        fpsHistory.push_back(fps);
    }
    fpsTimer += deltaTime;

    ImGui::Begin("Performance Info", nullptr, ImGuiWindowFlags_NoDecoration | ImGuiWindowFlags_AlwaysAutoResize);

    ImGui::TextColored(ImVec4(0.3f, 1.0f, 0.3f, 1.0f), "Performance");
    ImGui::Separator();
    ImGui::Text("FPS:        %.1f", fps);
    ImGui::Text("Delta Time: %.4f s", deltaTime);
    ImGui::Text("Frame Time: %.2f ms", deltaTime * 1000.0f);

    if (!fpsHistory.empty())
    {
        float maxFps = *std::max_element(fpsHistory.begin(), fpsHistory.end());
        maxFps = std::max(maxFps * 1.1f, 60.0f);

        ImGui::PlotLines("##FPS", fpsHistory.data(), fpsHistory.size(), 0, "FPS History",
                         0.0f, maxFps, ImVec2(-1, 50));
    }
    else
    {
        ImGui::Text("FPS data unavailable");
    }

    ImGui::Spacing();
    ImGui::TextColored(ImVec4(0.4f, 0.7f, 1.0f, 1.0f), "Renderer");
    ImGui::Separator();
    ImGui::Text("Draw Calls:     %d", Renderer::GetDrawCallCount());
    ImGui::Text("Lua OnUpdate Calls:   %d", m_OnUpdateCalls);
    ImGui::Text("Scene Lights:   %d / %d", Renderer::GetLightsCount(), g_engineConfig.gl_maxLight);
    ImGui::Text("Reserved Draws: %d", m_Reserved_draws);
    ImGui::Text("Camera Zoom:    %.2f", cameraZoom);
    ImGui::Text("Camera Pos:     (%.1f, %.1f)", cameraPos.x, cameraPos.y);

    ImGui::End();
}

void ShowColorCorrectionWindow()
{
    PROFILE_ENGINE_SCOPE("Engine::ShowColorCorrectionWindow");

    ColorCorrection *cc = Renderer::GetColorCorrection();
    if (!cc)
        return;

    ImGui::Begin("Color Correction");

    bool changed = false;
    changed |= ImGui::SliderFloat("Brightness", &cc->brightness, 0.0f, 2.0f, "%.2f");
    changed |= ImGui::SliderFloat("Saturation", &cc->saturation, 0.0f, 2.0f, "%.2f");
    changed |= ImGui::SliderFloat("Gamma", &cc->gamma, 0.1f, 4.0f, "%.2f");

    ImGui::SeparatorText("Bloom");

    changed |= ImGui::Checkbox("Enable Bloom", &cc->bloom);
    changed |= ImGui::SliderFloat("Threshold", &cc->threshold, 0.0f, 2.0f, "%.2f");
    changed |= ImGui::SliderFloat("Intensity", &cc->intensity, 0.0f, 4.0f, "%.2f");

    if (ImGui::Button("Reset to Default"))
    {
        *cc = ColorCorrection();
    }
    ImGui::SameLine();

    ImGui::End();
}

void Engine::Init()
{

    if (DeleteLatestLogFile())
    {
        Logger::LogVerbose("Log file deleted");
    }
    else
    {
        Logger::LogVerbose("Failed to delete log file");
    }

    glfwInit();
    glfwWindowHint(GLFW_CONTEXT_VERSION_MAJOR, 3);
    glfwWindowHint(GLFW_CONTEXT_VERSION_MINOR, 3);
    glfwWindowHint(GLFW_OPENGL_PROFILE, GLFW_OPENGL_CORE_PROFILE);

    window = glfwCreateWindow(1280, 720, "Create Editor", nullptr, nullptr);
    glfwMakeContextCurrent(window);
    glfwSwapInterval(0); // No VSync
    glewInit();

    g_engineConfig.LoadFromFile();

    IMGUI_CHECKVERSION();
    ImGui::CreateContext();
    ImGuiIO &io = ImGui::GetIO();
    io.ConfigFlags |= ImGuiConfigFlags_DockingEnable;
    ImGui::StyleColorsDark();
    ImGui_ImplGlfw_InitForOpenGL(window, true);
    ImGui_ImplOpenGL3_Init((std::string("#version ") + g_engineConfig.gl_version).c_str());
    m_Reserved_draws = 0;

    Renderer::Init();

    auto obj = std::make_shared<Object>("Hello, Create");

    objects.push_back(obj);

    selected = obj;

    //{
    //    std::random_device rd;
    //    std::mt19937 rng(rd());
    //    std::uniform_real_distribution<float> lightRadius(1000.0f, 2000.0f);
    //    std::uniform_real_distribution<float> lightIntensity(1.0f, 3.0f);
    //    std::uniform_real_distribution<float> hueDist(0.0f, 1.0f);
    //
    //    const float tileSize = 1024.0f; // approximate texture size
    //    const int gridSize = 10;
    //
    //    // 10x10 grid of pebble sprites
    //    for (int y = 0; y < gridSize; ++y)
    //    {
    //        for (int x = 0; x < gridSize; ++x)
    //        {
    //            int index = y * gridSize + x;
    //            auto obj = std::make_shared<Object>("Pebble_" + std::to_string(index));
    //            obj->SetLocalPosition(glm::vec2(x * tileSize, y * tileSize));
    //
    //            auto sprite = obj->AddComponent<SpriteComponent>();
    //            sprite->SetTexture("C:/Users/spenc/OneDrive/Pictures/textures/ganges_river_pebbles_diff_1k.png");
    //            sprite->SetNormalMap("C:/Users/spenc/OneDrive/Pictures/textures/ganges_river_pebbles_nor_gl_1k.png");
    //            sprite->SetRenderType(SpriteComponent::RenderType::Lit);
    //
    //            objects.push_back(obj);
    //        }
    //    }
    //
    //    // 100 random lights
    //    std::uniform_real_distribution<float> posDist(0.0f, tileSize * gridSize);
    //
    //    for (int i = 0; i < 100; ++i)
    //    {
    //        auto obj = std::make_shared<Object>("Light_" + std::to_string(i));
    //        obj->SetLocalPosition(glm::vec2(posDist(rng), posDist(rng)));
    //
    //        auto light = obj->AddComponent<LightComponent>();
    //        light->SetRadius(lightRadius(rng));
    //        light->SetIntensity(lightIntensity(rng));
    //        light->SetColor(HSVtoRGB(hueDist(rng), 0.8f, 1.0f));
    //
    //        objects.push_back(obj);
    //    }
    //}

    Logger::LogVerbose("Resverving Objects");

    m_toDraw.reserve(1024);
    m_scriptUpdates.reserve(256);
    m_collectStack.reserve(1024);
    m_physicsUpdates.reserve(1024);

    Logger::LogInfo("Initialized Engine");
}
void DrawInspectorUI(std::shared_ptr<Object> selected)
{
    PROFILE_ENGINE_SCOPE("Engine::DrawInspectorUI");
    ImGui::Begin("Inspector");

    if (!selected)
    {
        ImGui::TextDisabled("No object selected.");
        ImGui::End();
        return;
    }

    // --- keep track of the name buffer ---
    static char nameBuffer[128] = "";
    static std::shared_ptr<Object> lastSelected = nullptr;
    if (selected != lastSelected)
    {
        strncpy(nameBuffer, selected->GetName().c_str(), sizeof(nameBuffer));
        nameBuffer[sizeof(nameBuffer) - 1] = '\0';
        lastSelected = selected;
    }

    // --- General ---
    ImGui::TextUnformatted("General");
    ImGui::Separator();

    // Name edit
    if (ImGui::InputText("Name", nameBuffer, sizeof(nameBuffer)))
        selected->SetName(nameBuffer);

    // Visibility toggle
    bool visible = selected->GetVisable();
    if (ImGui::Checkbox("Visible", &visible))
        selected->SetVisable(visible);

    // --- Transform ---
    ImGui::TextUnformatted("Transform");
    ImGui::Separator();

    glm::vec2 pos = selected->GetLocalPosition();
    if (ImGui::DragFloat2("Position", &pos.x, 0.1f))
        selected->SetLocalPosition(pos);

    float rotation = selected->GetLocalRotation();
    if (ImGui::DragFloat("Rotation", &rotation, 1.0f))
        selected->SetLocalRotation(rotation);

    int layer = selected->layer;
    if (ImGui::InputInt("Layer", &layer))
        selected->layer = layer;

    ImGui::Text("ID: %d", selected->uid.id);
    ImGui::Text("UID: %s", selected->uid.uuid.c_str());

    ImGui::Spacing();
    ImGui::SeparatorText("Add Component");

    static const char *componentTypes[] = {
        "SpriteComponent",
        "CameraComponent",
        "LightComponent",
        "ScriptComponent",
        "TilemapComponent"};

    static int selectedIndex = -1;

    if (ImGui::BeginCombo("##ComponentCombo", selectedIndex >= 0 ? componentTypes[selectedIndex] : "Select Component"))
    {
        for (int i = 0; i < IM_ARRAYSIZE(componentTypes); ++i)
        {
            bool isSelected = (selectedIndex == i);
            if (ImGui::Selectable(componentTypes[i], isSelected))
                selectedIndex = i;
            if (isSelected)
                ImGui::SetItemDefaultFocus();
        }
        ImGui::EndCombo();
    }

    ImGui::SameLine();
    if (ImGui::Button("Add") && selectedIndex != -1 && selected)
    {
        const std::string type = componentTypes[selectedIndex];

        if (type == "SpriteComponent" && !selected->GetComponent<SpriteComponent>())
            selected->AddComponent<SpriteComponent>();
        else if (type == "CameraComponent" && !selected->GetComponent<CameraComponent>())
            selected->AddComponent<CameraComponent>();
        else if (type == "LightComponent" && !selected->GetComponent<LightComponent>())
            selected->AddComponent<LightComponent>();
        else if (type == "ScriptComponent" && !selected->GetComponent<ScriptComponent>())
            selected->AddComponent<ScriptComponent>();
        else if (type == "TilemapComponent" && !selected->GetComponent<TilemapComponent>())
            selected->AddComponent<TilemapComponent>();
    }

    if (auto sprite = selected->GetComponent<SpriteComponent>())
    {
        ImGui::SeparatorText("Sprite Component");

        const char *renderTypes[] = {"Unlit", "Lit"};
        SpriteComponent::RenderType currentType = sprite->GetRenderType();
        int currentTypeIndex = static_cast<int>(currentType);

        if (ImGui::Combo("Render Type", &currentTypeIndex, renderTypes, IM_ARRAYSIZE(renderTypes)))
        {
            sprite->SetRenderType(static_cast<SpriteComponent::RenderType>(currentTypeIndex));
        }

        std::string tex = sprite->GetTexturePath();
        ImGui::Text("Texture: %s", tex.c_str());
        if (ImGui::Button("Load Texture"))
        {
            auto path = OpenFileDialog(FileDialogType::Images);
            if (!path.empty())
                sprite->SetTexture(path);
        }

        std::string norm = sprite->GetNormalMapPath();
        ImGui::Text("Normal Map: %s", norm.c_str());
        if (ImGui::Button("Load Normal Map"))
        {
            auto path = OpenFileDialog(FileDialogType::Images);
            if (!path.empty())
                sprite->SetNormalMap(path);
        }

        glm::vec2 size = sprite->GetSize();
        ImGui::Text("Size: %.0f x %.0f", size.x, size.y);

        if (ImGui::Button("Remove SpriteComponent"))
            selected->RemoveComponent<SpriteComponent>();
    }

    if (auto cam = selected->GetComponent<CameraComponent>())
    {
        ImGui::SeparatorText("Camera Component");

        float fov = cam->GetFOV();
        float aspect = cam->GetAspect();
        float zoom = cam->GetZoom();

        if (ImGui::DragFloat("FOV", &fov, 0.1f, 1.0f, 179.0f))
            cam->SetFOV(fov);
        if (ImGui::DragFloat("Aspect", &aspect, 0.01f, 0.1f, 5.0f))
            cam->SetAspect(aspect);
        if (ImGui::DragFloat("Zoom", &zoom, 0.1f, 0.1f, 10.0f))
            cam->SetZoom(zoom);

        bool isPrimary = cam->IsPrimary();
        if (ImGui::Checkbox("Primary", &isPrimary))
            cam->SetPrimary(isPrimary);

        if (ImGui::Button("Remove CameraComponent"))
            selected->RemoveComponent<CameraComponent>();
    }

    if (auto light = selected->GetComponent<LightComponent>())
    {
        ImGui::SeparatorText("Light Component");

        glm::vec3 color = light->GetColor();
        float intensity = light->GetIntensity();
        float radius = light->GetRadius();
        float falloff = light->GetFalloff();
        int type = light->GetType();

        if (ImGui::ColorEdit3("Color", &color.x))
            light->SetColor(color);
        if (ImGui::DragFloat("Intensity", &intensity, 0.05f, 0.0f, 10.0f))
            light->SetIntensity(intensity);
        if (ImGui::DragFloat("Radius", &radius, 1.0f, 10.0f, 1000.0f))
            light->SetRadius(radius);
        if (ImGui::DragFloat("Falloff", &falloff, 0.05f, 0.1f, 5.0f))
            light->SetFalloff(falloff);

        // const char* types[] = { "Point", "Directional" };
        // if (ImGui::Combo("Type", &type, types, IM_ARRAYSIZE(types)))
        //     light->SetType(type);

        if (ImGui::Button("Remove LightComponent"))
            selected->RemoveComponent<LightComponent>();
    }
    if (auto script = selected->GetComponent<ScriptComponent>())
    {
        ImGui::SeparatorText("Script Component");

        std::string path = script->GetScriptPath();
        ImGui::Text("Script Path:");
        ImGui::TextWrapped("%s", path.c_str());

        if (ImGui::Button("Load Script"))
        {
            std::string chosen = OpenFileDialog(FileDialogType::Scripts); // Add 'Scripts' filter if needed
            if (!chosen.empty())
                script->SetScriptPath(chosen);
        }

        ImGui::SameLine();
        if (ImGui::Button("Reload"))
        {
            script->ReloadScript();
        }

        if (ImGui::Button("Remove ScriptComponent"))
            selected->RemoveComponent<ScriptComponent>();
    }
    if (auto phys = selected->GetComponent<PhysicsComponent>())
    {
        ImGui::SeparatorText("Physics Component");

        glm::vec2 vel = phys->GetVelocity();
        if (ImGui::DragFloat2("Velocity", &vel.x))
            phys->SetVelocity(vel);

        float friction = phys->GetFriction();
        if (ImGui::DragFloat("Friction", &friction, 0.01f, 0.0f, 1.0f))
            phys->SetFriction(friction);

        bool fixed = phys->IsFixedRotation();
        if (ImGui::Checkbox("Fixed Rotation", &fixed))
            phys->SetFixedRotation(fixed);

        if (ImGui::Button("Remove PhysicsComponent"))
            selected->RemoveComponent<PhysicsComponent>();
    }

    if (auto tilemap = selected->GetComponent<TilemapComponent>())
    {
        ImGui::SeparatorText("Tilemap Component");

        ImGui::Text("Refer to Tilemap Editor Window");

        if (ImGui::Button("Remove TilemapComponent"))
            selected->RemoveComponent<TilemapComponent>();
    }

    ImGui::End();

    if (auto tilemap = selected->GetComponent<TilemapComponent>())
    {
        tilemap->DrawEditorUI();
    }
}

void Engine::collectObjects(bool playing, const glm::vec2 &camPos, float camZoom)
{
    m_toDraw.clear();
    m_scriptUpdates.clear();
    m_collectStack.clear();
    m_activeCamera = nullptr;
    m_physicsUpdates.clear();

    const glm::vec2 screenSize = glm::vec2(Renderer::GetSize());

    for (auto &root : objects)
        if (!root->GetParent())
            m_collectStack.push_back(root);

    while (!m_collectStack.empty())
    {
        auto obj = m_collectStack.back();
        m_collectStack.pop_back();

        if (!obj->GetVisable())
            continue;

        m_toDraw.push_back(obj.get());

        if (auto light = obj->GetComponent<LightComponent>())
        {
            glm::vec2 world = obj->GetWorldPosition();
            glm::vec2 screen = (world - camPos) * camZoom + screenSize * 0.5f;
            Renderer::AddLight(screen, light->GetColor(), light->GetIntensity(), light->GetRadius() * camZoom);
        }

        if (!m_activeCamera)
        {
            if (auto camera = obj->GetComponent<CameraComponent>())
            {
                if (camera->IsPrimary())
                    m_activeCamera = camera;
            }
        }

        if (playing)
        {
            if (auto script = obj->GetComponent<ScriptComponent>())
                m_scriptUpdates.push_back(script.get());

            if (auto physics = obj->GetComponent<PhysicsComponent>())
                m_physicsUpdates.push_back(physics.get());
        }

        for (auto &child : obj->GetChildren())
            m_collectStack.push_back(child);
    }
}

void Engine::Run()
{
    while (!glfwWindowShouldClose(window))
    {
        if (g_engineConfig.settings.profile_editor)
        {
            profiler.BeginFrame();
        }

        profiler.BeginEngineSection("glfwPollEvents");

        glfwPollEvents();
        profiler.EndEngineSection();

        profiler.BeginEngineSection("NewFrame");

        profiler.BeginEngineSection("ImGui_ImplOpenGL3_NewFrame");

        ImGui_ImplOpenGL3_NewFrame();

        profiler.EndEngineSection();

        profiler.BeginEngineSection("ImGui_ImplGlfw_NewFrame");

        ImGui_ImplGlfw_NewFrame();

        profiler.EndEngineSection();

        profiler.BeginEngineSection("ImGui::NewFrame");

        ImGui::NewFrame();

        profiler.EndEngineSection();

        profiler.BeginEngineSection("ImGui::DockSpaceOverViewport");

        ImGui::DockSpaceOverViewport(ImGui::GetMainViewport()->ID);

        profiler.EndEngineSection();

        profiler.BeginEngineSection("glfwGetTime");

        float currentTime = glfwGetTime();
        static float lastTime = currentTime;
        float deltaTime = currentTime - lastTime;
        lastTime = currentTime;
        profiler.EndEngineSection();

        profiler.EndEngineSection();

        profiler.BeginEngineSection("Engine::ShowDebugOverlay");

        ShowDebugOverlay(deltaTime);
        profiler.EndEngineSection();

        profiler.BeginEngineSection("Logger::Draw");

        Logger::Draw();
        profiler.EndEngineSection();

        profiler.BeginEngineSection("BeginMainMenuBar");

        if (ImGui::BeginMainMenuBar())
        {
            // — left‑aligned items —
            if (ImGui::Button(playing ? "Stop" : "Play"))
            {
                if (!playing)
                {
                    Logger::LogVerbose("[RestoreScene] Saving original scene");
                    SaveState();
                }
                else
                {
                    Logger::LogVerbose("[RestoreScene] Reloading original scene");
                    LoadState();
                }
                selected = nullptr;
                playing = !playing;
            }

            if (ImGui::BeginMenu("File"))
            {
                if (ImGui::MenuItem("Save Scene"))
                {
                    std::string file = OpenFileDialog(FileDialogType::Scenes);
                    if (!file.empty())
                        SaveScene(file);
                    selected = nullptr;
                }
                if (ImGui::MenuItem("Load Scene"))
                {
                    std::string file = OpenFileDialog(FileDialogType::Scenes);
                    Logger::LogInfo("[LoadScene] Loading Scene.");
                    if (!file.empty())
                        LoadScene(file);
                    selected = nullptr;
                }
                ImGui::EndMenu();
            }

            if (ImGui::BeginMenu("Options"))
            {

                if (ImGui::BeginMenu("Settings"))
                {
                    ImGui::Checkbox("Enable Lighting", &g_engineConfig.settings.lighting_enabled);
                    ImGui::Checkbox("Enable Gizmos", &g_engineConfig.settings.draw_gizmos);
                    ImGui::Checkbox("Profiling", &g_engineConfig.settings.profile_enabled);
                    if (g_engineConfig.settings.profile_enabled)
                    {
                        ImGui::Checkbox("Profile Engine", &g_engineConfig.settings.profile_editor);
                        ImGui::Checkbox("Deep Profileing", &g_engineConfig.settings.profile_deep);
                    }
                    ImGui::EndMenu();
                }

                if (ImGui::BeginMenu("Windows"))
                {
                    ImGui::Checkbox("Color Correction", &g_engineConfig.settings.show_color_correction_window);
                    ImGui::EndMenu();
                }

                ImGui::EndMenu();
            }

            // — right‑aligned version text —
            const char *version_fmt = "Create Engine v%s";
            // build the text
            char buf[64];
            std::snprintf(buf, sizeof(buf), version_fmt, g_engineConfig.version.c_str());

            // calculate width of the text
            float textWidth = ImGui::CalcTextSize(buf).x;
            // move cursor to far right minus padding
            float avail = ImGui::GetWindowSize().x - ImGui::GetCursorPosX();
            ImGuiStyle &style = ImGui::GetStyle();
            ImGui::SameLine(avail - textWidth - style.FramePadding.x);

            ImGui::TextUnformatted(buf);

            ImGui::EndMainMenuBar();
        }
        profiler.EndEngineSection();

        if (g_engineConfig.settings.show_color_correction_window)
            ShowColorCorrectionWindow();

        {
            PROFILE_ENGINE_SCOPE("Engine::DrawSceneTree");

            ImGui::Begin("Scene Tree");

            if (ImGui::BeginPopupContextWindow("SceneTreeContext", ImGuiPopupFlags_MouseButtonRight))
            {
                if (ImGui::MenuItem("Create New"))
                {
                    auto obj = std::make_shared<Object>("NewObject");
                    objects.push_back(obj);
                    selected = obj;
                    ImGui::OpenPopup("RenameObject");
                }
                ImGui::Separator();
                if (ImGui::MenuItem("Create New Sprite"))
                {
                    auto obj = std::make_shared<Object>("NewSprite");
                    obj->AddComponent<SpriteComponent>();
                    objects.push_back(obj);
                    selected = obj;
                    ImGui::OpenPopup("RenameObject");
                }
                if (ImGui::MenuItem("Create New Light"))
                {
                    auto obj = std::make_shared<Object>("NewSprite");
                    obj->AddComponent<LightComponent>();
                    objects.push_back(obj);
                    selected = obj;
                    ImGui::OpenPopup("RenameObject");
                }
                ImGui::EndPopup();
            }

            for (auto &obj : objects)
                if (!obj->GetParent()) // Only draw root nodes
                    DrawObjectNode(obj);

            ImGui::End();
        }

        DrawInspectorUI(selected);

        // Viewport
        ImGui::Begin("Viewport", nullptr, ImGuiWindowFlags_NoScrollbar | ImGuiWindowFlags_NoScrollWithMouse);
        ImVec2 size = ImGui::GetContentRegionAvail();

        // Save/restore editor camera position/zoom on play toggle
        static bool lastPlaying = false;

        if (playing && !lastPlaying)
        {
            // Entering play mode – save editor camera
            editorCameraSavedPos = cameraPos;
            editorCameraSavedZoom = cameraZoom;
            editorCameraWasSaved = true;
        }
        else if (!playing && lastPlaying)
        {
            // Exiting play mode – restore editor camera
            if (editorCameraWasSaved)
            {
                cameraPos = editorCameraSavedPos;
                cameraZoom = editorCameraSavedZoom;
                editorCameraWasSaved = false;
            }
        }

        lastPlaying = playing;

        // Use active camera when playing
        if (playing && m_activeCamera)
        {
            cameraPos = m_activeCamera->GetOwner()->GetWorldPosition();
            cameraZoom = m_activeCamera->GetZoom();
        }

        // Use editor camera logic when not playing
        if (!playing && ImGui::IsWindowHovered())
        {
            // Pan
            if (ImGui::IsMouseDragging(ImGuiMouseButton_Right))
            {
                ImVec2 delta = ImGui::GetMouseDragDelta(ImGuiMouseButton_Right);
                ImGui::ResetMouseDragDelta(ImGuiMouseButton_Right);
                cameraPos -= glm::vec2(delta.x, delta.y) / cameraZoom;
            }

            // Zoom
            float wheel = ImGui::GetIO().MouseWheel;
            if (wheel != 0.0f)
            {
                float zoomFactor = (wheel > 0) ? 1.1f : 1.0f / 1.1f;

                ImVec2 mousePos = ImGui::GetMousePos();
                ImVec2 windowPos = ImGui::GetWindowPos();
                ImVec2 contentMin = ImGui::GetWindowContentRegionMin();

                glm::vec2 relativeMouse = glm::vec2(
                    mousePos.x - windowPos.x - contentMin.x,
                    mousePos.y - windowPos.y - contentMin.y);

                glm::vec2 before = (relativeMouse - glm::vec2(size.x / 2, size.y / 2)) / cameraZoom + cameraPos;

                cameraZoom *= zoomFactor;
                cameraZoom = std::clamp(cameraZoom, 0.1f, 10.0f);

                glm::vec2 after = (relativeMouse - glm::vec2(size.x / 2, size.y / 2)) / cameraZoom + cameraPos;

                cameraPos += before - after;
            }
        }

        // Resize
        profiler.BeginEngineSection("Renderer::Resize");
        Renderer::Resize((int)size.x, (int)size.y);
        profiler.EndEngineSection();

        // Begin
        profiler.BeginEngineSection("Renderer::Begin");
        Renderer::Begin();
        profiler.EndEngineSection();

        profiler.BeginEngineSection("Reserve");
        m_toDraw.reserve(m_Reserved_draws);
        m_Reserved_draws = 0;
        m_toDraw.clear();
        m_scriptUpdates.clear();
        profiler.EndEngineSection();

        profiler.BeginEngineSection("Draw Editor Grid");
        Renderer::DrawEditorGrid(cameraPos, cameraZoom);
        profiler.EndEngineSection();

        // Collect Objects
        profiler.BeginEngineSection("Collect Objects");
        collectObjects(playing, cameraPos, cameraZoom);
        profiler.EndEngineSection();

        profiler.BeginEngineSection("Renderer::UpdateClusterLights");

        Renderer::UpdateClusterLights();
        profiler.EndEngineSection();

        // Sort Objects
        profiler.BeginEngineSection("Sort Objects");
        if (m_toDraw.size() > 1)
        {
            std::sort(m_toDraw.begin(), m_toDraw.end(),
                      [](auto const &a, auto const &b)
                      {
                          if (a->layer != b->layer)
                              return a->layer < b->layer;
                          return a->GetWorldPosition().y < b->GetWorldPosition().y;
                      });
        }
        profiler.EndEngineSection();

        if (!g_engineConfig.settings.profile_editor)
            profiler.BeginFrame();

        m_OnUpdateCalls = 0;

        profiler.BeginSection("Script Updates");
        for (auto *script : m_scriptUpdates)
        {
            profiler.BeginSection("Script: " + script->GetOwner()->GetName());
            script->OnUpdate(deltaTime);
            m_OnUpdateCalls++;
            profiler.EndSection();
        }
        profiler.EndSection();

        profiler.BeginSection("Render");
        for (auto *obj : m_toDraw)
        {
            if (auto spritePtr = obj->GetComponent<SpriteComponent>())
            {
                profiler.BeginSection("Draw Sprite: " + obj->GetName());
                Renderer::DrawSprite(spritePtr.get(),
                                     obj->GetWorldPosition(),
                                     cameraZoom,
                                     cameraPos);
                profiler.EndSection();
            }
        }
        profiler.EndSection();

        // Finish frame profiling
        if (!g_engineConfig.settings.profile_editor)
            profiler.EndFrame();

        // End renderer
        profiler.BeginEngineSection("Renderer::End");
        Renderer::End();
        profiler.EndEngineSection();

        // Display render target texture
        GLuint texID = Renderer::GetFinalTexture();
        ImGui::Image((ImTextureID)(uintptr_t)texID, size, ImVec2(0, 1), ImVec2(1, 0));

        ImGui::End();

        if (ImGui::BeginPopup("RenameObject"))
        {
            static char nameBuffer[128];
            static bool once = true;

            if (once && selected)
            {
                strcpy(nameBuffer, selected->GetName().c_str());
                once = false;
            }

            ImGui::InputText("##rename", nameBuffer, sizeof(nameBuffer));
            if (ImGui::IsKeyPressed(ImGuiKey_Enter) || ImGui::Button("OK"))
            {
                if (selected)
                    selected->SetName(nameBuffer);
                ImGui::CloseCurrentPopup();
                once = true;
            }

            ImGui::SameLine();
            if (ImGui::Button("Cancel"))
            {
                ImGui::CloseCurrentPopup();
                once = true;
            }

            ImGui::EndPopup();
        }

        if (g_engineConfig.settings.profile_editor)
        {
            profiler.EndFrame(); // Finish frame
        }
        // ImGui render
        ShowProfilerTimeline();

        ImGui::Render();
        int w, h;
        glfwGetFramebufferSize(window, &w, &h);
        glViewport(0, 0, w, h);
        glClear(GL_COLOR_BUFFER_BIT);
        ImGui_ImplOpenGL3_RenderDrawData(ImGui::GetDrawData());
        glfwSwapBuffers(window);

        for (auto &obj : pendingDeletion)
        {
            Logger::LogVerbose("Deleting Object From Que: '%s', [%d, %s]", obj->GetName().c_str(), obj->uid.id, obj->uid.uuid.c_str());
            if (obj->GetParent())
            {
                obj->GetParent()->RemoveChild(obj.get());
            }
            else
            {
                objects.erase(std::remove_if(objects.begin(), objects.end(),
                                             [&](const std::shared_ptr<Object> &o)
                                             { return o == obj; }),
                              objects.end());
            }
        }
        pendingDeletion.clear();
    }
}

void Engine::DrawObjectNode(const std::shared_ptr<Object> &obj)
{
    PROFILE_ENGINE_SCOPE("Engine::DrawObjectNode");

    ImGuiTreeNodeFlags flags = ImGuiTreeNodeFlags_OpenOnArrow |
                               ImGuiTreeNodeFlags_SpanAvailWidth |
                               (obj == selected ? ImGuiTreeNodeFlags_Selected : 0);

    bool open = ImGui::TreeNodeEx((void *)(intptr_t)obj->uid.id, flags, "%s", obj->GetName().c_str());

    if (ImGui::IsItemClicked())
        selected = obj;

    // === Context Menu ===
    if (ImGui::BeginPopupContextItem())
    {
        if (ImGui::MenuItem("Rename"))
        {
            selected = obj;
            ImGui::OpenPopup("RenameObject");
        }
        if (ImGui::MenuItem("Delete"))
        {
            pendingDeletion.push_back(obj);
        }
        if (ImGui::MenuItem("Create Child"))
        {
            auto child = std::make_shared<Object>("NewObject");
            obj->AddChild(child);
            selected = child;
            ImGui::OpenPopup("RenameObject");
        }
        ImGui::EndPopup();
    }

    // === Drag Source ===
    if (ImGui::BeginDragDropSource())
    {
        ImGui::SetDragDropPayload("OBJECT", &obj, sizeof(obj));
        ImGui::Text("Move: %s", obj->GetName().c_str());
        ImGui::EndDragDropSource();
    }

    // === Drop Target ===
    if (ImGui::BeginDragDropTarget())
    {
        if (const ImGuiPayload *payload = ImGui::AcceptDragDropPayload("OBJECT"))
        {
            auto dragged = *(std::shared_ptr<Object> *)payload->Data;

            if (dragged != obj && dragged)
            {
                Object *oldParent = dragged->GetParent();

                if (oldParent)
                {
                    oldParent->RemoveChild(dragged.get());

                    for (const auto &candidate : objects)
                    {
                        if (candidate.get() == oldParent)
                        {
                            pendingDeletion.push_back(candidate);
                            break;
                        }
                    }
                }
                else
                {
                    objects.erase(std::remove(objects.begin(), objects.end(), dragged), objects.end());
                }

                obj->AddChild(dragged);
            }
        }
        ImGui::EndDragDropTarget();
    }

    // === Children ===
    if (open)
    {
        for (auto &child : obj->GetChildren())
            DrawObjectNode(child);
        ImGui::TreePop();
    }
}

bool VerifySceneHash(const YAML::Node &root)
{
    if (!root["scene_hash"] || !root["objects"])
        return false;

    YAML::Emitter sceneOnly;
    sceneOnly << root["objects"];

    std::string sceneString = sceneOnly.c_str();
    unsigned char hash[SHA256_DIGEST_LENGTH];
    SHA256(reinterpret_cast<const unsigned char *>(sceneString.c_str()), sceneString.size(), hash);

    std::ostringstream hashHex;
    for (int i = 0; i < SHA256_DIGEST_LENGTH; ++i)
        hashHex << std::hex << std::setw(2) << std::setfill('0') << static_cast<int>(hash[i]);

    return hashHex.str() == root["scene_hash"].as<std::string>();
}

void Engine::SaveScene(const std::string &path)
{
    YAML::Emitter out;
    YAML::Emitter sceneData;

    // Serialize object list only
    sceneData << YAML::BeginSeq;
    for (const auto &obj : objects)
        obj->Save(sceneData);
    sceneData << YAML::EndSeq;

    std::string objectData = sceneData.c_str();
    unsigned char hash[SHA256_DIGEST_LENGTH];
    SHA256(reinterpret_cast<const unsigned char *>(objectData.c_str()), objectData.size(), hash);

    std::ostringstream hashHex;
    for (int i = 0; i < SHA256_DIGEST_LENGTH; ++i)
        hashHex << std::hex << std::setw(2) << std::setfill('0') << static_cast<int>(hash[i]);

    // Build final full YAML output
    out << YAML::BeginMap;
    out << YAML::Key << "engine_version" << YAML::Value << g_engineConfig.version;
    out << YAML::Key << "scene_name" << YAML::Value << std::filesystem::path(path).stem().string();
    out << YAML::Key << "scene_hash" << YAML::Value << hashHex.str();
    out << YAML::Key << "format_version" << YAML::Value << 1;
    out << YAML::Key << "objects" << YAML::Value << YAML::Load(objectData);

    Logger::LogVerbose("[SaveScene] Saving Color Correction Attributes");

    out << YAML::Key << "color_correction" << YAML::BeginMap;
    out << YAML::Key << "brightness" << YAML::Value << Renderer::GetColorCorrection()->brightness;
    out << YAML::Key << "saturation" << YAML::Value << Renderer::GetColorCorrection()->saturation;
    out << YAML::Key << "gamma" << YAML::Value << Renderer::GetColorCorrection()->gamma;

    out << YAML::Key << "bloom" << YAML::Value << Renderer::GetColorCorrection()->bloom;
    out << YAML::Key << "intensity" << YAML::Value << Renderer::GetColorCorrection()->intensity;
    out << YAML::Key << "threshold" << YAML::Value << Renderer::GetColorCorrection()->threshold;

    out << YAML::EndMap;

    out << YAML::EndMap;

    std::ofstream file(path);
    file << out.c_str();
}

void Engine::LoadScene(const std::string &path)
{
    Logger::LogDebug("[LoadScene] Reading Scene File.");

    YAML::Node root = YAML::LoadFile(path);

    Logger::LogDebug("[LoadScene] Verifying Scene");

    if (!root["engine_version"] || !root["format_version"] || !root["scene_name"])
    {
        Logger::LogError("[LoadScene] Missing required metadata!");
        return;
    }

    if (root["engine_version"].as<std::string>() != "0.1.0")
    {
        Logger::LogError("[LoadScene] Version mismatch! Expected 0.1.0, got %s", root["engine_version"].as<std::string>().c_str());
        return;
    }

    if (!VerifySceneHash(root))
    {
        Logger::LogWarning("[LoadScene] Scene hash does not match! File may be corrupted or tampered.");
    }

    Logger::LogDebug("[LoadScene] Reseting Scene.");
    objects.clear();

    Logger::LogDebug("[LoadScene] Recreating Objects");

    const auto &objectArray = root["objects"];
    for (const auto &node : objectArray)
    {
        auto obj = std::make_shared<Object>("[DefaultObject]");
        obj->Load(node);
        objects.push_back(obj);
    }

    if (root["color_correction"])
    {
        Logger::LogVerbose("[LoadScene] Loading Color Correction Attributes");

        ColorCorrection *cc = Renderer::GetColorCorrection();
        const auto &data = root["color_correction"];
        if (cc)
        {
            if (data["brightness"])
                cc->brightness = data["brightness"].as<float>();
            if (data["saturation"])
                cc->saturation = data["saturation"].as<float>();
            if (data["gamma"])
                cc->gamma = data["gamma"].as<float>();

            if (data["bloom"])
                cc->bloom = data["bloom"].as<bool>();
            if (data["intensity"])
                cc->intensity = data["intensity"].as<float>();
            if (data["threshold"])
                cc->threshold = data["threshold"].as<float>();
        }
    }

    Logger::LogInfo("[LoadScene] Loaded scene: %s", root["scene_name"].as<std::string>().c_str());
}
void Engine::SaveState()
{
    YAML::Emitter out;
    out << YAML::BeginMap;

    out << YAML::Key << "objects" << YAML::BeginSeq;
    for (const auto &obj : objects)
        obj->Save(out);
    out << YAML::EndSeq;

    Logger::LogVerbose("[SaveState] Saving Color Correction State");

    out << YAML::Key << "color_correction" << YAML::BeginMap;
    out << YAML::Key << "brightness" << YAML::Value << Renderer::GetColorCorrection()->brightness;
    out << YAML::Key << "saturation" << YAML::Value << Renderer::GetColorCorrection()->saturation;
    out << YAML::Key << "gamma" << YAML::Value << Renderer::GetColorCorrection()->gamma;

    out << YAML::Key << "bloom" << YAML::Value << Renderer::GetColorCorrection()->bloom;
    out << YAML::Key << "intensity" << YAML::Value << Renderer::GetColorCorrection()->intensity;
    out << YAML::Key << "threshold" << YAML::Value << Renderer::GetColorCorrection()->threshold;

    out << YAML::EndMap;

    out << YAML::EndMap;

    savedStateYAML = out.c_str();
    Logger::LogVerbose("[SaveState] Scene serialized (%zu bytes)", savedStateYAML.size());
}
void Engine::LoadState()
{
    if (savedStateYAML.empty())
    {
        Logger::LogWarning("[LoadState] No scene state in memory.");
        return;
    }

    YAML::Node root;
    try
    {
        root = YAML::Load(savedStateYAML);
    }
    catch (const std::exception &e)
    {
        Logger::LogError("[LoadState] Failed to parse saved scene: %s", e.what());
        return;
    }

    objects.clear();

    const auto &objectArray = root["objects"];
    for (const auto &node : objectArray)
    {
        auto obj = std::make_shared<Object>("[DefaultObject]");
        obj->Load(node);
        objects.push_back(obj);
    }

    if (root["color_correction"])
    {
        Logger::LogVerbose("[LoadState] Loading Color Correction State");

        ColorCorrection *cc = Renderer::GetColorCorrection();
        const auto &data = root["color_correction"];
        if (cc)
        {
            if (data["brightness"])
                cc->brightness = data["brightness"].as<float>();
            if (data["saturation"])
                cc->saturation = data["saturation"].as<float>();
            if (data["gamma"])
                cc->gamma = data["gamma"].as<float>();

            if (data["bloom"])
                cc->bloom = data["bloom"].as<bool>();
            if (data["intensity"])
                cc->intensity = data["intensity"].as<float>();
            if (data["threshold"])
                cc->threshold = data["threshold"].as<float>();
        }
    }

    Logger::LogVerbose("[LoadState] Scene restored");
}

void Engine::Shutdown()
{
    g_engineConfig.SaveToFile();

    ImGui_ImplOpenGL3_Shutdown();
    ImGui_ImplGlfw_Shutdown();
    ImGui::DestroyContext();
    glfwDestroyWindow(window);
    glfwTerminate();

    std::filesystem::remove(tempScenePath);
}
